@@ -113,7 +113,7 @@ import freemarker.template.utility.UndeclaredThrowableException;
  */
 public class BeansWrapper implements ObjectWrapper
 {
-    public static final Object CAN_NOT_UNWRAP = new Object();
+    static final Object CAN_NOT_UNWRAP = new Object();
     private static final Class BIGINTEGER_CLASS = java.math.BigInteger.class;
     private static final Class BOOLEAN_CLASS = Boolean.class;
     private static final Class CHARACTER_CLASS = Character.class;
@@ -222,7 +222,7 @@ public class BeansWrapper implements ObjectWrapper
 
     private ObjectWrapper outerIdentity = this;
     private boolean simpleMapWrapper;
-    private boolean strict = false; 
+    private boolean strict = false;
     
     /**
      * Creates a new instance of BeansWrapper. The newly created instance
@@ -534,25 +534,51 @@ public class BeansWrapper implements ObjectWrapper
      * method is the inverse of the {@link #wrap(Object)} method. In addition
      * it will unwrap arbitrary {@link TemplateNumberModel} instances into
      * a number, arbitrary {@link TemplateDateModel} instances into a date,
-     * {@link TemplateScalarModel} instances into a String, and
-     * {@link TemplateBooleanModel} instances into a Boolean.
-     * All other objects are returned unchanged.
+     * {@link TemplateScalarModel} instances into a String, arbitrary 
+     * {@link TemplateBooleanModel} instances into a Boolean, arbitrary 
+     * {@link TemplateHashModel} instances into a Map, arbitrary 
+     * {@link TemplateSequenceModel} into a List, and arbitrary 
+     * {@link TemplateCollectionModel} into a Set. All other objects are 
+     * returned unchanged.
+     * @throws TemplateModelException if an attempted unwrapping fails.
      */
     public Object unwrap(TemplateModel model) throws TemplateModelException
     {
         return unwrap(model, OBJECT_CLASS);
     }
     
+    /**
+     * Attempts to unwrap a model into an object of the desired class. 
+     * Generally, this method is the inverse of the {@link #wrap(Object)} 
+     * method. It recognizes a wide range of hint classes - all Java built-in
+     * primitives, primitive wrappers, numbers, dates, sets, lists, maps, and
+     * native arrays.
+     * @param model the model to unwrap
+     * @param hint the class of the unwrapped result
+     * @return the unwrapped result of the desired class
+     * @throws TemplateModelException if an attempted unwrapping fails.
+     */
     public Object unwrap(TemplateModel model, Class hint) 
     throws TemplateModelException
+    {
+        final Object obj = unwrapInternal(model, hint);
+        if(obj == CAN_NOT_UNWRAP) {
+          throw new TemplateModelException("Can not unwrap model of type " + 
+              model.getClass().getName() + " to type " + hint.getName());
+        }
+        return obj;
+    }
+    
+    Object unwrapInternal(TemplateModel model, Class hint) 
+    throws TemplateModelException
     {
         return unwrap(model, hint, null);
     }
-    
+
     private Object unwrap(TemplateModel model, Class hint, Map recursionStops) 
     throws TemplateModelException
     {
-        if(model == nullModel) {
+        if(model == null || model == nullModel) {
             return null;
         }
         
@@ -1123,89 +1149,60 @@ public class BeansWrapper implements ObjectWrapper
             MethodDescriptor[] mda = beanInfo.getMethodDescriptors();
 
             for(int i = pda.length - 1; i >= 0; --i) {
-                PropertyDescriptor pd = pda[i];
-                if(pd instanceof IndexedPropertyDescriptor) {
-                    IndexedPropertyDescriptor ipd = 
-                        (IndexedPropertyDescriptor)pd;
-                    Method readMethod = ipd.getIndexedReadMethod();
-                    Method publicReadMethod = getAccessibleMethod(readMethod, 
-                            accessibleMethods);
-                    if(publicReadMethod != null && isSafeMethod(publicReadMethod)) {
-                        try {
-                            if(readMethod != publicReadMethod) {
-                                ipd = new IndexedPropertyDescriptor(
-                                        ipd.getName(), ipd.getReadMethod(), 
-                                        ipd.getWriteMethod(), publicReadMethod, 
-                                        ipd.getIndexedWriteMethod());
-                            }
-                            classMap.put(ipd.getName(), ipd);
-                            getArgTypes(classMap).put(publicReadMethod, 
-                                    publicReadMethod.getParameterTypes());
-                        }
-                        catch(IntrospectionException e) {
-                            logger.warn("Failed creating a publicly-accessible " +
-                                    "property descriptor for " + clazz.getName() + 
-                                    " indexed property " + pd.getName() + 
-                                    ", read method " + publicReadMethod + 
-                                    ", write method " + ipd.getIndexedWriteMethod(), 
-                                    e);
-                        }
-                    }
-                }
-                else {
-                    Method readMethod = pd.getReadMethod();
-                    Method publicReadMethod = getAccessibleMethod(readMethod, accessibleMethods);
-                    if(publicReadMethod != null && isSafeMethod(publicReadMethod)) {
-                        try {
-                            if(readMethod != publicReadMethod) {
-                                pd = new PropertyDescriptor(pd.getName(), 
-                                        publicReadMethod, pd.getWriteMethod());
-                                pd.setReadMethod(publicReadMethod);
-                            }
-                            classMap.put(pd.getName(), pd);
-                        }
-                        catch(IntrospectionException e) {
-                            logger.warn("Failed creating a publicly-accessible " +
-                                    "property descriptor for " + clazz.getName() + 
-                                    " property " + pd.getName() + ", read method " + 
-                                    publicReadMethod + ", write method " + 
-                                    pd.getWriteMethod(), e);
-                        }
-                    }
-                }
+                populateClassMapWithPropertyDescriptor(
+                        pda[i], clazz, accessibleMethods,
+                        classMap);
             }
             if(exposureLevel < EXPOSE_PROPERTIES_ONLY)
             {
+                MethodAppearanceDecision decision = new MethodAppearanceDecision();  
                 for(int i = mda.length - 1; i >= 0; --i)
                 {
                     MethodDescriptor md = mda[i];
-                    Method method = md.getMethod();
-                    Method publicMethod = getAccessibleMethod(method, accessibleMethods);
+                    Method publicMethod = getAccessibleMethod(
+                            md.getMethod(), accessibleMethods);
                     if(publicMethod != null && isSafeMethod(publicMethod))
                     {
-                        String name = md.getName();
-                        Object previous = classMap.get(name);
-                        if(previous instanceof Method)
+                        decision.setDefaults(publicMethod);
+                        finetuneMethodAppearance(clazz, publicMethod, decision);
+                        
+                        PropertyDescriptor propDesc = decision.getExposeAsProperty();
+                        if (propDesc != null
+                                && !(classMap.get(propDesc.getName())
+                                        instanceof PropertyDescriptor))
                         {
-                            // Overloaded method - replace method with a method map
-                            MethodMap methodMap = new MethodMap(name, this);
-                            methodMap.addMember((Method)previous);
-                            methodMap.addMember(publicMethod);
-                            classMap.put(name, methodMap);
-                            // remove parameter type information
-                            getArgTypes(classMap).remove(previous);
+                            populateClassMapWithPropertyDescriptor(
+                                    propDesc, clazz, accessibleMethods,
+                                    classMap);
                         }
-                        else if(previous instanceof MethodMap)
+                        
+                        String methodKey = decision.getExposeMethodAs();
+                        if (methodKey != null)
                         {
-                            // Already overloaded method - add new overload
-                            ((MethodMap)previous).addMember(publicMethod);
-                        }
-                        else
-                        {
-                            // Simple method (this far)
-                            classMap.put(name, publicMethod);
-                            getArgTypes(classMap).put(publicMethod, 
-                                    publicMethod.getParameterTypes());
+                            Object previous = classMap.get(methodKey);
+                            if(previous instanceof Method)
+                            {
+                                // Overloaded method - replace method with a method map
+                                MethodMap methodMap = new MethodMap(methodKey, this);
+                                methodMap.addMember((Method)previous);
+                                methodMap.addMember(publicMethod);
+                                classMap.put(methodKey, methodMap);
+                                // remove parameter type information
+                                getArgTypes(classMap).remove(previous);
+                            }
+                            else if(previous instanceof MethodMap)
+                            {
+                                // Already overloaded method - add new overload
+                                ((MethodMap)previous).addMember(publicMethod);
+                            }
+                            else if (decision.getMethodShadowsProperty()
+                                    || !(previous instanceof PropertyDescriptor))
+                            {
+                                // Simple method (this far)
+                                classMap.put(methodKey, publicMethod);
+                                getArgTypes(classMap).put(publicMethod, 
+                                        publicMethod.getParameterTypes());
+                            }
                         }
                     }
                 }
@@ -1219,6 +1216,117 @@ public class BeansWrapper implements ObjectWrapper
         }
     }
 
+    private void populateClassMapWithPropertyDescriptor(PropertyDescriptor pd,
+            Class clazz, Map accessibleMethods, Map classMap) {
+        if(pd instanceof IndexedPropertyDescriptor) {
+            IndexedPropertyDescriptor ipd = 
+                (IndexedPropertyDescriptor)pd;
+            Method readMethod = ipd.getIndexedReadMethod();
+            Method publicReadMethod = getAccessibleMethod(readMethod, 
+                    accessibleMethods);
+            if(publicReadMethod != null && isSafeMethod(publicReadMethod)) {
+                try {
+                    if(readMethod != publicReadMethod) {
+                        ipd = new IndexedPropertyDescriptor(
+                                ipd.getName(), ipd.getReadMethod(), 
+                                ipd.getWriteMethod(), publicReadMethod, 
+                                ipd.getIndexedWriteMethod());
+                    }
+                    classMap.put(ipd.getName(), ipd);
+                    getArgTypes(classMap).put(publicReadMethod, 
+                            publicReadMethod.getParameterTypes());
+                }
+                catch(IntrospectionException e) {
+                    logger.warn("Failed creating a publicly-accessible " +
+                            "property descriptor for " + clazz.getName() + 
+                            " indexed property " + pd.getName() + 
+                            ", read method " + publicReadMethod + 
+                            ", write method " + ipd.getIndexedWriteMethod(), 
+                            e);
+                }
+            }
+        }
+        else {
+            Method readMethod = pd.getReadMethod();
+            Method publicReadMethod = getAccessibleMethod(readMethod, accessibleMethods);
+            if(publicReadMethod != null && isSafeMethod(publicReadMethod)) {
+                try {
+                    if(readMethod != publicReadMethod) {
+                        pd = new PropertyDescriptor(pd.getName(), 
+                                publicReadMethod, pd.getWriteMethod());
+                        pd.setReadMethod(publicReadMethod);
+                    }
+                    classMap.put(pd.getName(), pd);
+                }
+                catch(IntrospectionException e) {
+                    logger.warn("Failed creating a publicly-accessible " +
+                            "property descriptor for " + clazz.getName() + 
+                            " property " + pd.getName() + ", read method " + 
+                            publicReadMethod + ", write method " + 
+                            pd.getWriteMethod(), e);
+                }
+            }
+        }
+    }
+    
+    /**
+     * <b>Experimental method; subject to change!</b>
+     * Override this to tweak certain aspects of how methods appear in the
+     * data-model. {@link BeansWrapper} will pass in all Java methods here that
+     * it intends to expose in the data-model as methods (so you can do
+     * <tt>obj.foo()</tt> in the template). By default this method does nothing.
+     * By overriding it you can do the following tweaks:
+     * <ul>
+     *   <li>Hide a method that would be otherwise shown by calling
+     *     {@link MethodAppearanceDecision#setExposeMethodAs(String)}
+     *     with <tt>null</tt> parameter. Note that you can't un-hide methods
+     *     that are not public or are considered to by unsafe
+     *     (like {@link Object#wait()}) because
+     *     {@link #finetuneMethodAppearance} is not called for those.</li>
+     *   <li>Show the method with a different name in the data-model than its
+     *     real name by calling
+     *     {@link MethodAppearanceDecision#setExposeMethodAs(String)}
+     *     with non-<tt>null</tt> parameter.
+     *   <li>Create a fake JavaBean property for this method by calling
+     *     {@link MethodAppearanceDecision#setExposeAsProperty(PropertyDescriptor)}.
+     *     For example, if you have <tt>int size()</tt> in a class, but you
+     *     want it to be accessed from the templates as <tt>obj.size</tt>,
+     *     rather than as <tt>obj.size()</tt>, you can do that with this.
+     *     The default is {@code null}, which means that no fake property is
+     *     created for the method. You need not and shouldn't set this
+     *     to non-<tt>null</tt> for the getter methods of real JavaBean
+     *     properties, as those are automatically shown as properties anyway.
+     *     The property name in the {@link PropertyDescriptor} can be anything,
+     *     but the method (or methods) in it must belong to the class that
+     *     is given as the <tt>clazz</tt> parameter or it must be inherited from
+     *     that class, or else whatever errors can occur later.
+     *     {@link IndexedPropertyDescriptor}-s are supported.
+     *     If a real JavaBean property of the same name exists, it won't be
+     *     replaced by the fake one. Also if a fake property of the same name
+     *     was assigned earlier, it won't be replaced.
+     *   <li>Prevent the method to hide a JavaBean property (fake or real) of
+     *     the same name by calling
+     *     {@link MethodAppearanceDecision#setMethodShadowsProperty(boolean)}
+     *     with <tt>false</tt>. The default is <tt>true</tt>, so if you have
+     *     both a property and a method called "foo", then in the template
+     *     <tt>myObject.foo</tt> will return the method itself instead
+     *     of the property value, which is often undesirable.
+     * </ul>
+     * 
+     * <p>Note that you can expose a Java method both as a method and as a
+     * JavaBean property on the same time, however you have to chose different
+     * names for them to prevent shadowing. 
+     * 
+     * @param decision Stores how the parameter method will be exposed in the
+     *   data-model after {@link #finetuneMethodAppearance} returns.
+     *   This is initialized so that it reflects the default
+     *   behavior of {@link BeansWrapper}.
+     */
+    protected void finetuneMethodAppearance(
+            Class clazz, Method m, MethodAppearanceDecision decision) {
+        // left everything on its default; do nothing
+    }
+
     private static Map getArgTypes(Map classMap) {
         Map argTypes = (Map)classMap.get(ARGTYPES);
         if(argTypes == null) {
@@ -1574,4 +1682,46 @@ public class BeansWrapper implements ObjectWrapper
             return false;
         }
     }
+    
+    /**
+     * <b>Experimental class; subject to change!</b>
+     * Used for {@link #finetuneMethodAppearance} as output parameter; see there.
+     */
+    static public final class MethodAppearanceDecision {
+        private PropertyDescriptor exposeAsProperty;
+        private String exposeMethodAs;
+        private boolean methodShadowsProperty;
+        
+        void setDefaults(Method m) {
+            exposeAsProperty = null;
+            exposeMethodAs = m.getName();
+            methodShadowsProperty = true;
+        }
+        
+        public PropertyDescriptor getExposeAsProperty() {
+            return exposeAsProperty;
+        }
+        
+        public void setExposeAsProperty(PropertyDescriptor exposeAsProperty) {
+            this.exposeAsProperty = exposeAsProperty;
+        }
+        
+        public String getExposeMethodAs() {
+            return exposeMethodAs;
+        }
+        
+        public void setExposeMethodAs(String exposeAsMethod) {
+            this.exposeMethodAs = exposeAsMethod;
+        }
+        
+        public boolean getMethodShadowsProperty() {
+            return methodShadowsProperty;
+        }
+        
+        public void setMethodShadowsProperty(boolean shadowEarlierProperty) {
+            this.methodShadowsProperty = shadowEarlierProperty;
+        }
+
+    }
+
 }
