@@ -297,6 +297,13 @@ public class Configuration extends Configurable implements Cloneable {
         return cache.getTemplateLoader();
     }
 
+    /**
+     * Sets the {@link CacheStorage} used for caching {@link Template}-s. The
+     * default is a {@link SoftCacheStorage}. If the total size of the {@link Template}
+     * objects is significant but most templates are used rarely, using a
+     * {@link MruCacheStorage} instead might be advisable. If you don't want caching at
+     * all, use {@link freemarker.cache.NullCacheStorage} (you can't use <tt>null</tt>).
+     */
     public synchronized void setCacheStorage(CacheStorage storage) {
         createTemplateCache(cache.getTemplateLoader(), storage);
     }
@@ -437,13 +444,12 @@ public class Configuration extends Configurable implements Cloneable {
     }
     
     /**
-     * @return whether the alternative square bracket
-     * syntax is set as the default
+     * See {@link #setTagSyntax(int)} to see the returned number.
      */
     public int getTagSyntax() {
         return tagSyntax;
     }
-
+    
     /**
      * Equivalent to <tt>getTemplate(name, thisCfg.getLocale(), thisCfg.getEncoding(thisCfg.getLocale()), true)</tt>.
      */
@@ -703,7 +709,7 @@ public class Configuration extends Configurable implements Cloneable {
      *       </table>
      *       The value is not case sensitive. The order of <tt>soft</tt> and <tt>strong</tt>
      *       entries is not significant.
-     *       See also: {@link #setCacheStorage}
+     *       For more details see: {@link #setCacheStorage}
      *   <li><code>"template_update_delay"</code>: Valid positive integer, the
      *       update delay measured in seconds.
      *       See: {@link #setTemplateUpdateDelay}
@@ -725,6 +731,7 @@ public class Configuration extends Configurable implements Cloneable {
         } else if ("DefaultEncoding".equalsIgnoreCase(key)) {
             key = DEFAULT_ENCODING_KEY;
         }
+        boolean callSuper = false;
         try {
             if (DEFAULT_ENCODING_KEY.equals(key)) {
                 setDefaultEncoding(value);
@@ -769,9 +776,9 @@ public class Configuration extends Configurable implements Cloneable {
             } else if (TEMPLATE_UPDATE_DELAY_KEY.equals(key)) {
                 setTemplateUpdateDelay(Integer.parseInt(value));
             } else if (AUTO_INCLUDE_KEY.equals(key)) {
-                setAutoIncludes(new SettingStringParser(value).parseAsList());
+                setAutoIncludes(parseAsList(value));
             } else if (AUTO_IMPORT_KEY.equals(key)) {
-                setAutoImports(new SettingStringParser(value).parseAsImportList());
+                setAutoImports(parseAsImportList(value));
             } else if (TAG_SYNTAX_KEY.equals(key)) {
                 if ("auto_detect".equals(value)) {
                     setTagSyntax(AUTO_DETECT_TAG_SYNTAX);
@@ -783,13 +790,18 @@ public class Configuration extends Configurable implements Cloneable {
                     throw invalidSettingValueException(key, value);
                 }
             } else {
-                super.setSetting(key, value);
+                callSuper = true;
             }
         } catch(Exception e) {
             throw new TemplateException(
-                    "Failed to set setting " + key + " to value " + value,
+                    "Failed to set setting " + 
+                    StringUtil.jQuote(key) + " to value " + StringUtil.jQuote(value) +
+                    "; see cause exception.",
                     e, getEnvironment());
         }
+        if (callSuper) {
+            super.setSetting(key, value);
+        }
     }
     
     
