@@ -63,16 +63,19 @@ import java.io.PrintWriter;
 import java.io.Reader;
 import java.io.StringWriter;
 import java.io.Writer;
+import java.math.BigDecimal;
+import java.math.BigInteger;
 import java.net.URL;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.GregorianCalendar;
 import java.util.HashMap;
-import java.util.Locale;
 import java.util.Map;
 import java.util.ResourceBundle;
+import java.util.Set;
 import java.util.StringTokenizer;
 import java.util.TimeZone;
+import java.util.TreeSet;
 
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
@@ -87,13 +90,13 @@ import freemarker.ext.beans.ResourceBundleModel;
 import freemarker.ext.dom.NodeModel;
 import freemarker.ext.jdom.NodeListModel;
 import freemarker.template.Configuration;
-import freemarker.template.ObjectWrapper;
 import freemarker.template.SimpleCollection;
 import freemarker.template.SimpleDate;
 import freemarker.template.SimpleNumber;
 import freemarker.template.Template;
 import freemarker.template.TemplateBooleanModel;
 import freemarker.template.TemplateDateModel;
+import freemarker.template.TemplateException;
 import freemarker.template.TemplateMethodModel;
 import freemarker.template.TemplateNodeModel;
 import freemarker.template.TemplateScalarModel;
@@ -165,48 +168,17 @@ public class TemplateTestCase extends TestCase {
         else if ("outputdir".equals(param)) {
             setOutputDirectory(value);
         }
-        else if ("output_encoding".equals(param)) {
-            conf.setOutputEncoding(value);
-        }
-        else if ("locale".equals(param)) {
-            String lang = "", country="", variant="";
-            StringTokenizer st = new StringTokenizer(value,"_", false);
-            if (st.hasMoreTokens()) {
-                lang = st.nextToken();
-            }
-            if (st.hasMoreTokens()) {
-                country = st.nextToken();
-            }
-            if (st.hasMoreTokens()){
-                variant = st.nextToken();
-            }
-            if (lang != "") {
-                Locale loc = new Locale(lang, country, variant);
-                conf.setLocale(loc);
-            }
-        }
-        else if ("object_wrapper".equals(param)) {
+        else {
             try {
-                Class cl = Class.forName(value);
-                ObjectWrapper ow = (ObjectWrapper) cl.newInstance();
-                conf.setObjectWrapper(ow);
-            } catch (Exception e) {
-                fail("Error setting object wrapper to " + value + "\n" + e.getMessage());
+                conf.setSetting(param, value);
+            } catch (TemplateException e) {
+                throw new RuntimeException(
+                        "Failed to set setting " +
+                        StringUtil.jQuote(param) + " to " +
+                        StringUtil.jQuote(value) + "; see cause exception.",
+                        e);
             }
         }
-        else if ("input_encoding".equals(param)) {
-            conf.setDefaultEncoding(value);
-        }
-        else if ("output_encoding".equals(param)) {
-            conf.setOutputEncoding(value);
-        }
-        else if ("strict_syntax".equals(param)) {
-            boolean b = StringUtil.getYesNo(value);
-            conf.setStrictSyntaxMode(b);
-        }
-        else if ("url_escaping_charset".equals(param)) {
-            conf.setURLEscapingCharset(value);
-        }
     }
     
     /*
@@ -328,7 +300,7 @@ public class TemplateTestCase extends TestCase {
             dataModel.put("doc", new NodeListModel(doc));
         }
         
-        else if (testName.equals("string-builtins3")) {
+        else if (testName.equals("test-stringbimethods")) {
             dataModel.put("multi", new TestBoolean());
         }
         
@@ -369,12 +341,34 @@ public class TemplateTestCase extends TestCase {
             NodeModel nm = NodeModel.parse(is);
             dataModel.put("doc", nm);
         }
-        
         else if (testName.equals("xmlns5")) {
             InputSource is = new InputSource(getClass().getResourceAsStream("test-defaultxmlns1.xml"));
             NodeModel nm = NodeModel.parse(is);
             dataModel.put("doc", nm);
         }
+        
+        else if (testName.startsWith("sequence-builtins-with-")) {
+            Set abcSet = new TreeSet();
+            abcSet.add("a");
+            abcSet.add("b");
+            abcSet.add("c");
+            dataModel.put("abcSet", abcSet);
+        }
+        
+        else if (testName.equals("test-iso8601")) {
+            dataModel.put("javaGMT02", TimeZone.getTimeZone("GMT+02"));
+            dataModel.put("javaUTC", TimeZone.getTimeZone("UTC"));
+            dataModel.put("adaptedToStringScalar", new Object() {
+                public String toString() {
+                    return "GMT+02";
+                }
+            });
+        }
+        
+        else if (testName.equals("test-number-to-date")) {
+          dataModel.put("bigInteger", new BigInteger("1305575275540"));
+          dataModel.put("bigDecimal", new BigDecimal("1305575275539.5"));
+        }
     }
     
     public void runTest() {
